@@ -60,6 +60,9 @@ Device::Device(QObject* parent, const QString& id)
     //Register in bus
     QDBusConnection::sessionBus().registerObject(dbusPath(), this, QDBusConnection::ExportScriptableContents | QDBusConnection::ExportAdaptors);
 
+    //Assume every plugin is supported until addLink is called and we can get the actual list
+    m_supportedPlugins = PluginLoader::instance()->getPluginList().toSet();
+
     connect(this, &Device::pairingError, this, &warn);
 }
 
@@ -94,7 +97,7 @@ QStringList Device::loadedPlugins() const
 
 void Device::reloadPlugins()
 {
-    QHash<QString, KdeConnectPlugin*> newPluginMap;
+    QHash<QString, KdeConnectPlugin*> newPluginMap, oldPluginMap = m_plugins;
     QMultiMap<QString, KdeConnectPlugin*> newPluginsByIncomingCapability;
 
     if (isTrusted() && isReachable()) { //Do not load any plugin for unpaired devices, nor useless loading them for unreachable devices
@@ -124,7 +127,7 @@ void Device::reloadPlugins()
         }
     }
 
-    const bool differentPlugins = m_plugins != newPluginMap;
+    const bool differentPlugins = oldPluginMap != newPluginMap;
 
     //Erase all left plugins in the original map (meaning that we don't want
     //them anymore, otherwise they would have been moved to the newPluginMap)
@@ -259,7 +262,6 @@ void Device::removeLink(DeviceLink* link)
     //qCDebug(KDECONNECT_CORE) << "RemoveLink" << m_deviceLinks.size() << "links remaining";
 
     if (m_deviceLinks.isEmpty()) {
-        m_supportedPlugins.clear();
         reloadPlugins();
         Q_EMIT reachableStatusChanged();
     }
@@ -401,18 +403,21 @@ bool Device::isPluginEnabled(const QString& pluginName) const
 QString Device::encryptionInfo() const
 {
     QString result;
+    QCryptographicHash::Algorithm digestAlgorithm = QCryptographicHash::Algorithm::Sha1;
 
-    QString myCertificate = QString::fromLatin1(KdeConnectConfig::instance()->certificate().toDer());
-    for (int i=2 ; i<myCertificate.size() ; i+=3) {
-        myCertificate.insert(i, ':'); // Improve readability
+    QString localSha1 = QString::fromLatin1(KdeConnectConfig::instance()->certificate().digest(digestAlgorithm).toHex());
+    for (int i=2 ; i<localSha1.size() ; i+=3) {
+        localSha1.insert(i, ':'); // Improve readability
     }
-    result += i18n("SHA1 fingerprint of your device certificate is: %1\n", myCertificate);
+    result += i18n("SHA1 fingerprint of your device certificate is: %1\n", localSha1);
 
-    QString remoteCertificate = KdeConnectConfig::instance()->getDeviceProperty(id(), "certificate");
-    for (int i=2 ; i<remoteCertificate.size() ; i+=3) {
-        remoteCertificate.insert(i, ':'); // Improve readability
+    std::string  remotePem = KdeConnectConfig::instance()->getDeviceProperty(id(), "certificate").toStdString();
+    QSslCertificate remoteCertificate = QSslCertificate(QByteArray(remotePem.c_str(), remotePem.size()));
+    QString remoteSha1 = QString::fromLatin1(remoteCertificate.digest(digestAlgorithm).toHex());
+    for (int i=2 ; i<remoteSha1.size() ; i+=3) {
+        remoteSha1.insert(i, ':'); // Improve readability
     }
-    result += i18n("SHA1 fingerprint of remote device certificate is: %1\n", remoteCertificate);
+    result += i18n("SHA1 fingerprint of remote device certificate is: %1\n", remoteSha1);
 
     return result;
 }
