@@ -83,6 +83,10 @@ KdeConnectKcm::KdeConnectKcm(QWidget *parent, const QVariantList&)
 
     kcmUi->rename_label->setText(daemon->announcedName());
     kcmUi->rename_edit->setText(daemon->announcedName());
+    connect(daemon, SIGNAL(announcedNameChanged(QString)),
+            kcmUi->rename_edit, SLOT(setText(QString)));
+    connect(daemon, SIGNAL(announcedNameChanged(QString)),
+            kcmUi->rename_label, SLOT(setText(QString)));
     setRenameMode(false);
 
     setButtons(KCModule::Help | KCModule::NoAdditionalButton);
@@ -220,20 +224,16 @@ void KdeConnectKcm::resetDeviceView()
 
     const QList<KPluginInfo> pluginInfo = KPluginInfo::fromMetaData(KPluginLoader::findPlugins("kdeconnect/"));
     QList<KPluginInfo> availablePluginInfo;
-    QList<KPluginInfo> unsupportedPluginInfo;
 
     m_oldSupportedPluginNames = currentDevice->supportedPlugins();
     for (auto it = pluginInfo.cbegin(), itEnd = pluginInfo.cend(); it!=itEnd; ++it) {
         if (m_oldSupportedPluginNames.contains(it->pluginName())) {
             availablePluginInfo.append(*it);
-        } else {
-            unsupportedPluginInfo.append(*it);
         }
     }
 
     KSharedConfigPtr deviceConfig = KSharedConfig::openConfig(currentDevice->pluginsConfigFile());
     kcmUi->pluginSelector->addPlugins(availablePluginInfo, KPluginSelector::ReadConfigFile, i18n("Available plugins"), QString(), deviceConfig);
-    kcmUi->pluginSelector->addPlugins(unsupportedPluginInfo, KPluginSelector::ReadConfigFile, i18n("Unavailable plugins"), QString(), deviceConfig);
     connect(kcmUi->pluginSelector, SIGNAL(changed(bool)), this, SLOT(pluginsConfigChanged()));
 
 }
