@@ -24,23 +24,14 @@
 #include <QtCore/QThread>
 #include <QDBusMetaType>
 
-#include <KDebug>
-#include <KComponentData>
-#include <KCmdLineArgs>
-#include <KAboutData>
-#include <KProcess>
-#include <KApplication>
-#include <KLocale>
+#include <KLocalizedString>
 
-#include <core/kdebugnamespace.h>
+#include <QDebug>
 
-extern "C" int KDE_EXPORT kdemain(int argc, char **argv)
+Q_LOGGING_CATEGORY(KDECONNECT_KIO, "kdeconnect.kio")
+
+extern "C" int Q_DECL_EXPORT kdemain(int argc, char **argv)
 {
-    KAboutData about("kiokdeconnect", "kdeconnect-kio", ki18n("kiokdeconnect"), "1.0");
-    KCmdLineArgs::init(&about);
-
-    KApplication app;
-
     if (argc != 4) {
         fprintf(stderr, "Usage: kio_kdeconnect protocol pool app\n");
         exit(-1);
@@ -74,7 +65,7 @@ bool handleDBusError(QDBusReply<T>& reply, KIO::SlaveBase* slave)
 {
     if (!reply.isValid())
     {
-        kDebug(debugArea()) << "Error in DBus request:" << reply.error();
+        qCDebug(KDECONNECT_KIO) << "Error in DBus request:" << reply.error();
         slave->error(toKioError(reply.error().type()),reply.error().message());
         return true;
     }
@@ -95,16 +86,13 @@ void KioKdeconnect::listAllDevices()
     //TODO: Change to all devices and show different icons for connected and disconnected?
     QStringList devices = m_dbusInterface->devices(true, true);
 
-    totalSize(devices.length());
-
-    int i = 0;
     Q_FOREACH(const QString& deviceId, devices) {
 
         DeviceDbusInterface interface(deviceId);
 
         if (!interface.hasPlugin("kdeconnect_sftp")) continue;
 
-        const QString target = QString("kdeconnect://").append(deviceId).append("/");
+        const QString path = QString("kdeconnect://").append(deviceId).append("/");
         const QString name = interface.name();
         const QString icon = "kdeconnect";
 
@@ -112,16 +100,20 @@ void KioKdeconnect::listAllDevices()
         entry.insert(KIO::UDSEntry::UDS_NAME, name);
         entry.insert(KIO::UDSEntry::UDS_ICON_NAME, icon);
         entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
-        entry.insert(KIO::UDSEntry::UDS_ACCESS, S_IRUSR | S_IRGRP | S_IROTH);
+        entry.insert(KIO::UDSEntry::UDS_ACCESS, S_IRUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
         entry.insert(KIO::UDSEntry::UDS_MIME_TYPE, "");
-        entry.insert(KIO::UDSEntry::UDS_URL, target);
-        listEntry(entry, false);
-
-        processedSize(i++);
-
+        entry.insert(KIO::UDSEntry::UDS_URL, path);
+        listEntry(entry);
     }
 
-    listEntry(KIO::UDSEntry(), true);
+    // We also need a non-null and writable UDSentry for "."
+    KIO::UDSEntry entry;
+    entry.insert(KIO::UDSEntry::UDS_NAME, QStringLiteral("."));
+    entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
+    entry.insert(KIO::UDSEntry::UDS_SIZE, 0);
+    entry.insert(KIO::UDSEntry::UDS_ACCESS, S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IWGRP | S_IXGRP | S_IROTH | S_IXOTH);
+    listEntry(entry);
+
     infoMessage("");
     finished();
 }
@@ -130,46 +122,54 @@ void KioKdeconnect::listDevice()
 {
     infoMessage(i18n("Accessing device..."));
 
-    kDebug(debugArea()) << "ListDevice" << m_currentDevice;
+    qCDebug(KDECONNECT_KIO) << "ListDevice" << m_currentDevice;
 
     SftpDbusInterface interface(m_currentDevice);
-    
+
     QDBusReply<bool> mountreply = interface.mountAndWait();
-    
+
     if (handleDBusError(mountreply, this)) {
         return;
     }
-    
+
     if (!mountreply.value()) {
         error(KIO::ERR_COULD_NOT_MOUNT, i18n("Could not mount device filesystem"));
         return;
     }
-    
+
     QDBusReply< QVariantMap > urlreply = interface.getDirectories();
-    
+
     if (handleDBusError(urlreply, this)) {
         return;
     }
-    
-    QVariantMap urls = urlreply.value();
-    
-    for (QVariantMap::iterator it = urls.begin(); it != urls.end(); it++) {
 
-        QString path = it.key();
-        QString name = it.value().toString();
+    QVariantMap urls = urlreply.value();
+
+    for (QVariantMap::iterator it = urls.begin(); it != urls.end(); ++it) {
+
+        const QString path = it.key();
+        const QString name = it.value().toString();
+        const QString icon = QLatin1Literal("folder");
 
         KIO::UDSEntry entry;
         entry.insert(KIO::UDSEntry::UDS_NAME, "files");
         entry.insert(KIO::UDSEntry::UDS_DISPLAY_NAME, name);
-        entry.insert(KIO::UDSEntry::UDS_ICON_NAME, "folder");
+        entry.insert(KIO::UDSEntry::UDS_ICON_NAME, icon);
         entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
-        entry.insert(KIO::UDSEntry::UDS_ACCESS, S_IRUSR | S_IRGRP | S_IROTH);
+        entry.insert(KIO::UDSEntry::UDS_ACCESS, S_IRUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
         entry.insert(KIO::UDSEntry::UDS_MIME_TYPE, "");
-        entry.insert(KIO::UDSEntry::UDS_URL, path);
-        listEntry(entry, false);
+        entry.insert(KIO::UDSEntry::UDS_URL, QUrl::fromLocalFile(path).toString());
+        listEntry(entry);
     }
 
-    listEntry(KIO::UDSEntry(), true);
+    // We also need a non-null and writable UDSentry for "."
+    KIO::UDSEntry entry;
+    entry.insert(KIO::UDSEntry::UDS_NAME, QStringLiteral("."));
+    entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
+    entry.insert(KIO::UDSEntry::UDS_SIZE, 0);
+    entry.insert(KIO::UDSEntry::UDS_ACCESS, S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IWGRP | S_IXGRP | S_IROTH | S_IXOTH);
+    listEntry(entry);
+
     infoMessage("");
     finished();
 
@@ -177,17 +177,16 @@ void KioKdeconnect::listDevice()
 
 
 
-void KioKdeconnect::listDir(const KUrl &url)
+void KioKdeconnect::listDir(const QUrl &url)
 {
-    kDebug(debugArea()) << "Listing..." << url;
+    qCDebug(KDECONNECT_KIO) << "Listing..." << url;
 
-    /// Url is not used here becuase all we could care about the url is the host, and that's already
+    /// Url is not used here because all we could care about the url is the host, and that's already
     /// handled in @p setHost
     Q_UNUSED(url);
 
     if (!m_dbusInterface->isValid()) {
         infoMessage(i18n("Could not contact background service."));
-        listEntry(KIO::UDSEntry(), true);
         finished();
         return;
     }
@@ -199,9 +198,9 @@ void KioKdeconnect::listDir(const KUrl &url)
     }
 }
 
-void KioKdeconnect::stat(const KUrl &url)
+void KioKdeconnect::stat(const QUrl &url)
 {
-    kDebug(debugArea()) << "Stat: " << url;
+    qCDebug(KDECONNECT_KIO) << "Stat: " << url;
 
     KIO::UDSEntry entry;
     entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
@@ -210,9 +209,9 @@ void KioKdeconnect::stat(const KUrl &url)
     finished();
 }
 
-void KioKdeconnect::get(const KUrl &url)
+void KioKdeconnect::get(const QUrl &url)
 {
-    kDebug(debugArea()) << "Get: " << url;
+    qCDebug(KDECONNECT_KIO) << "Get: " << url;
     mimeType("");
     finished();
 }
@@ -222,7 +221,7 @@ void KioKdeconnect::setHost(const QString &hostName, quint16 port, const QString
 
     //This is called before everything else to set the file we want to show
 
-    kDebug(debugArea()) << "Setting host: " << hostName;
+    qCDebug(KDECONNECT_KIO) << "Setting host: " << hostName;
 
     // In this kio only the hostname is used
     Q_UNUSED(port)
